from . import gear


# Digital addressable lighting interface –
# Part 202: Particular requirements for control gear – Self-contained emergency
# lighting (device type 1)


class _EmergencyCommand(gear._StandardCommand):
    _devicetype = 1
    _sendtwice = True


class _EmergencyQuery(gear._StandardCommand):
    _devicetype = 1
    _response = gear.command.Response


class Rest(_EmergencyCommand):
    """
    If the command is received when the control gear is in emergency mode then the lamp shall
    be extinguished and the control gear shall go into rest mode.
    Rest mode shall revert to normal mode in the event of restoration of normal supply. If re-light
    in rest mode is supported the control gear shall revert to emergency mode on receipt of
    command 226 ‘RE-LIGHT/RESET INHIBIT’.
    No other command shall cause the lamp to be extinguished when the control gear is in
    emergency mode.

    """
    _cmdval = 224


class Inhibit(_EmergencyCommand):
    """
    If the control gear is in normal mode on receipt of this command, bit 0 of the EMERGENCY
    STATUS byte shall be set, the control gear shall go into inhibit mode and start a 15 min inhibit
    timer. If the control gear is in inhibit mode on receipt of this command, the 15 min inhibit timer
    shall be restarted.
    If the control gear is in any other mode when the command is received, it shall be ignored.
    Whilst the 15 min inhibit timer is running, the control gear shall be prevented from going into
    emergency mode. In the event of a mains failure during this period, the control gear shall go
    into rest mode.
    If mains power is still available, bit 0 of the EMERGENCY STATUS byte shall be cleared and
    the control gear shall return to normal mode if:
    − the 15 min inhibit timer period expires, or
    − command 226 ‘RE-LIGHT/RESET INHIBIT’ is received
    """
    _cmdval = 225


class ResetInhibit(_EmergencyCommand):
    """
    This command shall cancel the inhibit timer.
    If re-light in rest mode is supported, this command shall also put the control gear back into
    emergency mode when mains is not present unless further inhibited by the hardwired inhibit.
    NOTE 1 Re-light in rest mode will normally cause the lamp to be lit, and the battery will consequently be
    discharged.
    NOTE 2 The state transitions referred to in the descriptions of commands 224, 225 and 226 are shown in Figure
    2.
    """
    _cmdval = 226


class StartFunctionTest(_EmergencyCommand):
    """
    The command shall request the control gear to perform a function test.
    The command shall be ignored if a function test is already in progress. Otherwise the control
    gear shall proceed as follows.
    The control gear may delay the start of a function test if the battery charge level is low or the
    control gear is in any mode other than normal mode, but the test shall not be delayed for any
    other reason.
    If the control gear is not able to start the function test immediately, the function test shall be
    set as pending until it can be performed. The delay of the function test shall be flagged in bit 4
    of the EMERGENCY STATUS byte. If a delayed function test cannot be completed within the
    maximum delay time defined by the TEST EXECUTION TIMEOUT byte, this shall be flagged
    in bit 4 of the FAILURE STATUS byte, bit 4 of the EMERGENCY STATUS byte shall remain
    set and the test shall remain pending.
    When the function test starts, bit 4 of the EMERGENCY MODE byte shall be set and bit 1 and
    bit 4 of the EMERGENCY STATUS byte shall be cleared.
    On completion of a function test, bit 1 of the EMERGENCY STATUS byte shall be set. If the
    function test is failed because of a lamp fault, then bit 3 and bit 6 of the FAILURE STATUS
    byte shall be set. If the function test is failed either because of a battery fault or because
    there was insufficient battery capacity to complete the test, then bit 2 and bit 6 of the
    FAILURE STATUS byte shall be set.
    Bit 4 of the FAILURE STATUS byte shall be cleared if a function test is completed without the
    TEST EXECUTION TIMEOUT period having expired.
    NOTE A function test ends upon self-timeout or if a fault condition is detected.
    """
    _cmdval = 227


class StartDurationtest(_EmergencyCommand):
    """
    The command shall request the control gear to perform a duration test.
    The command shall be ignored if a duration test is already in progress. Otherwise the control
    gear shall then proceed as follows:
    If there is sufficient battery charge and the control gear is in normal mode, a duration test
    shall be started immediately. Otherwise, the duration test shall be set as pending until it can
    be performed. The delay of the duration test shall be flagged in bit 5 in the EMERGENCY
    STATUS byte. If the start of the duration test is delayed beyond the time implied by the TEST
    EXECUTION TIMEOUT byte, this shall be flagged in bit 5 of the FAILURE STATUS byte, bit 5
    of the EMERGENCY STATUS byte shall remain set and the test shall remain pending.
    NOTE It is the responsibility of the control system to ensure that this command is issued at a time when the
    consequent duration test will not compromise safety.
    When the duration test starts, bit 5 of the EMERGENCY MODE byte shall be set, bit 2 and bit
    5 of the EMERGENCY STATUS byte shall be cleared, and the DURATION TEST RESULT
    byte shall be set to zero. As the test proceeds, the DURATION TEST RESULT byte shall be
    incremented at appropriate intervals.
    On completion of the test, bit 2 of the EMERGENCY STATUS byte shall be set. If the test was
    successful, bit 7 of the FAILURE STATUS byte shall be cleared and the value of DURATION
    TEST RESULT byte shall be greater than or equal to the value of the RATED DURATION
    byte. If the test was not successful, bit 7 of the FAILURE STATUS byte shall be set and the
    DURATION TEST RESULT byte shall give an indication of the time for which the emergency
    control gear successfully powered the lamp from the battery supply.
    Bit 5 of the FAILURE STATUS byte shall be cleared if a duration test is completed without the
    TEST EXECUTION TIMEOUT period having expired.
    """

    _cmdval = 228


class StopTest(_EmergencyCommand):
    """
    On receipt of this command any pending tests shall be cancelled and bits 4 and 5 of the
    EMERGENCY STATUS byte shall be cleared. If the gear is performing either a function test or
    a duration test then that test shall be stopped and the gear shall return to normal mode.
    If the control gear is in any mode other than function test or duration test, the mode shall not
    be changed.
    NOTE 1 Neither a function test nor a duration test can be performed whilst mains is not present, so the specified
    return to normal mode will always be possible.
    NOTE 2 Bits 4 and 5 of the FAILURE STATUS are not affected by this command. In other words, if a test delay
    has already been exceeded, the information that this has occurred is not lost.
    """
    _cmdval = 229


class ResetFunctionTestDoneFlag(_EmergencyCommand):
    """
    The "function test done and result valid" flag (bit 1 of the EMERGENCY STATUS byte) shall
    be cleared.
    NOTE This flag indicates that a function test has been carried out completely, and that the function test result
    indicated in bit 6 of the FAILURE STATUS byte is valid.
    """
    _cmdval = 230


class ResetDurationTestDoneFlag(_EmergencyCommand):
    """
    The "duration test done and result valid" flag (bit 2 of the EMERGENCY STATUS byte) shall
    be cleared.
    NOTE This flag indicates that a duration test has been carried out completely, and that both the value in the
    DURATION TEST RESULT byte and the result indicated in bit 7 of the FAILURE STATUS byte are valid.
    """
    _cmdval = 231


class ResetLampTime(_EmergencyCommand):
    """
    The LAMP EMERGENCY TIME and the LAMP TOTAL OPERATION TIME counters shall be reset.
    """
    _cmdval = 232


class StoreDtrAsEmergencyLevel(_EmergencyCommand):
    """
    If the EMERGENCY LEVEL is adjustable, the data held in the Data Transfer Register shall be
    stored as the EMERGENCY LEVEL.
    Programming the EMERGENCY LEVEL shall not influence the actual arc power level in any
    mode.
    NOTE This command should only be used during the set-up procedure and can be password or key protected by
    the control system.
    """
    _cmdval = 233
    _uses_dtr0 = True


class StoreTestDelayTimeHighByte(_EmergencyCommand):
    """
    The data held in the Data Transfer Register shall be stored as the high byte of TEST DELAY
    TIME in “quarters of an hour”. This byte represents the high byte that together with command
    235 ‘STORE TEST DELAY TIME LOW BYTE’ forms a 16-bit FUNCTION TEST DELAY TIME
    or DURATION TEST DELAY TIME number in “quarters of an hour”.
    If automatic testing is not supported the command shall be ignored.
    """
    _cmdval = 234
    _uses_dtr0 = True


class StoreTestDelayTimeLowByte(_EmergencyCommand):
    """
    The data held in the Data Transfer Register shall be stored as the low byte of TEST DELAY
    TIME in "quarters of an hour". This byte represents the low byte that together with command
    234 ‘STORE TEST DELAY TIME HIGH BYTE’ forms a 16-bit FUNCTION TEST DELAY TIME
    or DURATION TEST DELAY TIME number in "quarters of an hour".
    If automatic testing is not supported the command shall be ignored.
    """
    _cmdval = 235
    _uses_dtr0 = True


class StoreFunctionTestInterval(_EmergencyCommand):
    """
    The data held in the Data Transfer Register shall be stored as the FUNCTION TEST
    INTERVAL.
    The FUNCTION TEST INTERVAL is defined in days (1 to 255). A function test as described
    under command 227 shall be initiated after the end of each interval period. A DTR value of 0
    shall disable the auto-function test.
    At the moment the command is received, with a DTR value larger than 0, the next function
    test shall be delayed by a time depending on the previously stored value of the TEST DELAY
    TIME HIGH BYTE and the TEST DELAY TIME LOW BYTE.
    Automatically initiated tests may be further delayed so as to be carried out at a time estimated
    to be of low risk.
    If automatic testing is not supported the command shall be ignored.
    """
    _cmdval = 236
    _uses_dtr0 = True


class StoreDurationTestInterval(_EmergencyCommand):
    """
    The data held in the Data Transfer Register shall be stored as the DURATION TEST
    INTERVAL.
    The DURATION TEST INTERVAL is defined in weeks (1 to 97). A duration test as described
    under command 228 shall be initiated after the end of each interval period. A DTR value of 0
    shall disable the auto duration test.
    At the moment the command is received, with a DTR value larger than 0, the next duration
    test shall be delayed by a time depending on the previously stored value of the TEST DELAY
    TIME HIGH BYTE and the TEST DELAY TIME LOW BYTE.
    Automatically initiated tests may be further delayed so as to be carried out at a time estimated
    to be of low risk.
    If automatic testing is not supported the command shall be ignored.
    """
    _cmdval = 237
    _uses_dtr0 = True


class StoreTestExecutionTimeout(_EmergencyCommand):
    """
    The data held in the Data Transfer Register shall be stored as the TEST EXECUTION
    TIMEOUT. The TEST EXECUTION TIMEOUT is defined in days (0 to 255).
    The TEST EXECUTION TIMEOUT shall be valid for all function and duration tests, whether
    attempted in response to a command or as a result of an automatic test schedule. If the TEST
    EXECUTION TIMEOUT is set to 0 this shall result in a test timeout interval of 15 min.
    The TEST EXECUTION TIMEOUT period shall start when a test becomes pending.
    If the TEST EXECUTION TIMEOUT period expires without the test being finished, it shall be
    flagged as a failure in the FAILURE STATUS byte (command 252, bits 4 and 5) but the test
    shall remain pending.
    NOTE Meaningful values for TEST EXECUTION TIMEOUT are 0 up to the test interval, or any value if automatic
    testing is disabled or not supported. The default value is 7 days. If a test is requested the test execution can be
    delayed if the control gear is not ready to test.
    """
    _cmdval = 238
    _uses_dtr0 = True


class StoreProlongTime(_EmergencyCommand):
    """
    The data held in the Data Transfer Register shall be stored as the PROLONG TIME.
    The PROLONG TIME is defined with a resolution of 0,5 min (0 to 255), and shall be used to
    determine the length of time the control gear shall remain in extended emergency mode. If the
    PROLONG TIME value is 0, then the control gear shall return to normal mode from
    emergency mode as soon as the mains is restored.
    The PROLONG TIME shall be recalled when the control gear enters the extended emergency
    mode. Changing the PROLONG TIME whilst the control gear is in extended emergency mode
    shall have no immediate effect on the timing.
    """
    _cmdval = 239
    _uses_dtr0 = True


class StartIdentification(_EmergencyCommand):
    """
    The control gear shall start or restart a ten-second identification procedure.
    NOTE 1 The details of this procedure can only be defined by the manufacturer.
    NOTE 2 An appropriate procedure might be to change the arc power so as to facilitate identification of the control
    gear by optical means.
    """
    _cmdval = 240


class QueryBatteryCharge(_EmergencyQuery):
    """
    The answer shall be an 8-bit value in the range 0 to 254 representing the actual battery
    charge level from the deep discharge point to fully charged. If the emergency control gear
    cannot perform this function “MASK” (255) shall be returned.
    NOTE The value for battery charge level may not be valid if the control gear has not performed a successful
    duration test
    """
    _cmdval = 241


class QueryTestTiming(_EmergencyQuery):
    """The answer depends on the content of DTR0:

    0 - time until the next function test in quarters of an hour (high byte)
    1 - time until the next function test in quarters of an hour (low byte)
    2 - time until the next duration test in quarters of an hour (high byte)
    3 - time until the next duration test in quarters of an hour (low byte)
    4 - function test interval time in days (or 0)
    5 - duration test interval time in weeks (or 0)
    6 - test execution timeout in days
    7 - prolong time in 30s intervals

    When the high byte of a 16-bit value is read, the low byte is
    placed in DTR1 to enable atomic reads.  If automatic testing is
    not supported then queries 0-3 will return 255 (MASK).
    """

    _cmdval = 242


class QueryDurationTestResult(_EmergencyQuery):
    """
    The answer shall be the duration test result as an 8-bit value in 120-s (2 min) steps. A value
    of 255 means the maximum value (510 min, 8 h, 30 min) or longer.
    """

    _cmdval = 243


class QueryLampEmergencyTime(_EmergencyQuery):
    """
    The answer shall be the accumulated lamp functioning time (‘LAMP EMERGENCY TIME’),
    with the battery as power source, as an 8-bit value in 1-h steps. A value of 255 means the
    maximum value of 254 h or longer. The timer shall be incremented at the beginning of the 1 h
    timeslot.
    NOTE 1 When the ‘LAMP EMERGENCY TIME’ reaches 255 h, it will stay at this value, until reset by command
    232. To get the correct time over more than 254 h, a control unit should read out the value, accumulate it and reset
    the timer with command 232.
    NOTE 2 Although the reported value is in units of 1 h, the control gear should accumulate the time internally with
    greater precision.
    """

    _cmdval = 244


class QueryLampTotalEmergencyTime(_EmergencyQuery):
    """
    The answer shall be the accumulated lamp total functioning time (‘LAMP TOTAL OPERATION
    TIME’) as an 8-bit value in 4-h steps. A value of 255 means the maximum value of 1 016 h or
    longer. The timer shall be incremented at the beginning of the 4 h timeslot.
    NOTE 1 When the value of ‘LAMP TOTAL OPERATION TIME’ reaches 255, it will stay at this value, until reset by
    command 232. To get the correct time over more than 1 016 h, a control unit should read out the value, accumulate
    it and reset the timer with command 232.
    NOTE 2 If the lamp is operated in normal mode by another control gear, the self-contained emergency control
    gear might not be able to detect the correct total lamp operation time.
    """
    _cmdval = 245


class QueryEmergencyLevel(_EmergencyQuery):
    """
    The answer shall be the EMERGENCY LEVEL as an 8-bit number. If the EMERGENCY
    LEVEL is unknown, MASK (255) shall be returned.
    """
    _cmdval = 246


class QueryEmergencyMinLevel(_EmergencyQuery):
    """
    The answer shall be the EMERGENCY MIN LEVEL as an 8-bit number. If the EMERGENCY
    MIN LEVEL is unknown, MASK (255) shall be returned.
    """
    _cmdval = 247


class QueryEmergencyMaxLevel(_EmergencyQuery):
    """
    The answer shall be the EMERGENCY MAX LEVEL as an 8-bit number. If the EMERGENCY
    MAX LEVEL is unknown, MASK (255) shall be returned.
    """
    _cmdval = 248


class QueryRatedDuration(_EmergencyQuery):
    """
    The answer shall be an 8-bit number representing the rated duration as a multiple of 2 min.
    """
    _cmdval = 249


class QueryEmergencyMode(_EmergencyQuery):
    """
    The answer shall be the following ‘EMERGENCY MODE INFORMATION’ byte
    """

    class Response(gear.command.BitmapResponse):
        bits = [
            'rest mode active',
            'normal mode active',
            'emergency mode active',
            'extended emergency mode active',
            'function test is in progress',
            'duration test is in progress',
            'hardwired inhibit is active',
            'hardwired switch is on',
        ]

    _cmdval = 250
    _response = Response


class QueryFeatures(_EmergencyQuery):
    """
    The answer shall be the following ‘FEATURES’ information byte describing the type of control gear
    """

    class Response(gear.command.BitmapResponse):
        bits = [
            'integral emergency control gear',
            'maintained control gear',
            'switched maintained control gear',
            'auto test capability',
            'adjustable emergency level',
            'hardwired inhibit supported',
            'physical selection supported',
            're-light in rest mode supported',
        ]

    _cmdval = 251
    _response = Response


class QueryFailureStatus(_EmergencyQuery):
    """
    The answer shall be the following ‘FAILURE STATUS’ information byte
    """

    class Response(gear.command.BitmapResponse):
        bits = [
            'circuit failure',
            'battery duration failure',
            'battery failure',
            'emergency lamp failure',
            'function test max. delay exceeded',
            'duration test max. delay exceeded',
            'function test failed',
            'duration test failed',
        ]

    _cmdval = 252
    _response = Response


class QueryEmergencyStatus(_EmergencyQuery):
    """
    The answer shall be the following ‘EMERGENCY STATUS’ information byte
    """

    class Response(gear.command.BitmapResponse):
        bits = [
            'inhibit mode',
            'function test done and result valid',
            'duration test done and result valid',
            'battery fully charged',
            'function test request pending',
            'duration test request pending',
            'identification active',
            'physically selected',
        ]

    _cmdval = 253
    _response = Response
